=== modified file 'NEWS'
--- NEWS	2008-03-17 21:48:18 +0000
+++ NEWS	2008-03-18 02:49:45 +0000
@@ -22,6 +22,9 @@
 
   BUGFIXES:
 
+    * ``bzr mv a b`` can be now used also to rename previously renamed
+      directories, not only files. (Lukáš Lalinský, #107967)
+
   DOCUMENTATION:
 
   API BREAKS:

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2008-03-14 19:44:24 +0000
+++ bzrlib/builtins.py	2008-03-17 21:58:47 +0000
@@ -534,13 +534,31 @@
         if len(names_list) < 2:
             raise errors.BzrCommandError("missing file argument")
         tree, rel_names = tree_files(names_list)
+        tree.lock_write()
+        try:
+            self._run(tree, names_list, rel_names, after)
+        finally:
+            tree.unlock()
 
-        dest = names_list[-1]
-        isdir = os.path.isdir(dest)
-        if (isdir and not tree.case_sensitive and len(rel_names) == 2
-            and rel_names[0].lower() == rel_names[1].lower()):
-                isdir = False
-        if isdir:
+    def _run(self, tree, names_list, rel_names, after):
+        into_existing = osutils.isdir(names_list[-1])
+        if into_existing and len(names_list) == 2:
+            # special cases:
+            # a. case-insensitive filesystem and change case of dir
+            # b. move directory after the fact (if the source used to be
+            #    a directory, but now doesn't exist in the working tree
+            #    and the target is an existing directory, just rename it)
+            if (not tree.case_sensitive
+                and rel_names[0].lower() == rel_names[1].lower()):
+                into_existing = False
+            else:
+                inv = tree.inventory
+                from_id = tree.path2id(rel_names[0])
+                if (not osutils.lexists(names_list[0]) and
+                    from_id and inv.get_file_kind(from_id) == "directory"):
+                    into_existing = False
+        # move/rename
+        if into_existing:
             # move into existing directory
             for pair in tree.move(rel_names[:-1], rel_names[-1], after=after):
                 self.outf.write("%s => %s\n" % pair)

=== modified file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	2008-03-03 23:04:47 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2008-03-05 13:09:32 +0000
@@ -407,3 +407,26 @@
         self.failUnlessExists('sub/a2')
         self.assertInWorkingTree('sub/a1')
         self.assertInWorkingTree('sub/a2')
+
+    def test_mv_already_moved_directory(self):
+        """Use `bzr mv a b` to mark a directory as renamed.
+
+        https://bugs.launchpad.net/bzr/+bug/107967/
+        """
+        self.build_tree(['a/', 'c/'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a', 'c'])
+        osutils.rename('a', 'b')
+        osutils.rename('c', 'd')
+        # mv a b should work just like it does for already renamed files
+        self.run_bzr('mv a b')
+        self.failIfExists('a')
+        self.assertNotInWorkingTree('a')
+        self.failUnlessExists('b')
+        self.assertInWorkingTree('b')
+        # and --after should work, too (technically it's ignored)
+        self.run_bzr('mv --after c d')
+        self.failIfExists('c')
+        self.assertNotInWorkingTree('c')
+        self.failUnlessExists('d')
+        self.assertInWorkingTree('d')

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2008-03-07 14:15:10 +0000
+++ bzrlib/workingtree_4.py	2008-03-17 21:58:47 +0000
@@ -662,8 +662,8 @@
         if self._inventory is not None:
             update_inventory = True
             inv = self.inventory
+            to_dir_id = to_entry[0][2]
             to_dir_ie = inv[to_dir_id]
-            to_dir_id = to_entry[0][2]
         else:
             update_inventory = False
 

